<?php


namespace App\Controller;


use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;

/**
 * @Route("/mean-price-for-squared-meter", name="mean_price_for_squared_meter")
 */
class MeanPriceForSquaredMeter extends AbstractController
{
    /**
     * Entity Manager
     */
    private $em;
    public function __construct(EntityManagerInterface $em) {
        $this->em = $em;
    }

    public function __invoke(Request $request) {
        $query = "  SELECT
                        TO_CHAR(valeurs_fonciere.date_mutation, 'YYYY') AS year,
                        TO_CHAR(valeurs_fonciere.date_mutation, 'MM') AS month,
                        AVG(valeurs_fonciere.valeur_fonciere / valeurs_fonciere.surface_reelle_bati) AS mean_metre_price
                    FROM
                        App:ValeursFonciere AS valeurs_fonciere
                    WHERE
                        valeurs_fonciere.surface_reelle_bati > 0
                        AND
                        valeurs_fonciere.valeur_fonciere > 0
                        AND
                        lower(valeurs_fonciere.nature_mutation) = 'vente'
                    GROUP BY 
                        year,
                        month
                    ORDER BY
                        year,
                        month";
        $metre_prices = $this->em->createQuery($query)->getResult();
        foreach ($metre_prices as &$metre_price){
            $metre_price['year'] = intval($metre_price['year']);
            $metre_price['month'] = intval($metre_price['month']);
            $metre_price['mean_metre_price'] = floatval($metre_price['mean_metre_price']);
        }
        $res = new JsonResponse(
            $metre_prices,
            Response::HTTP_OK
        );
        return $res;
    }
}
