<?php


namespace App\Controller;


use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;

/**
 * @Route("/number-of-sold-by-params/{interval}/{start_date}/{end_date}", name="number_of_sold_by_params")
 */
class NumberOfSoldByParams extends AbstractController
{
    /**
     * Entity Manager
     */
    private $em;
    public function __construct(EntityManagerInterface $em) {
        $this->em = $em;
    }

    private static $INTERVAL_DAY = 'day';
    private static $INTERVAL_MONTH = 'month';
    private static $INTERVAL_YEAR = 'year';

    public function __invoke(Request $request, $interval, $start_date, $end_date) {
        $selection = "TO_CHAR(valeurs_fonciere.date_mutation, 'YYYY') AS year";
        $groupByAndOrder = "year";
        if($interval == self::$INTERVAL_DAY){
            $selection .= ",
                TO_CHAR(valeurs_fonciere.date_mutation, 'MM') AS month,
                TO_CHAR(valeurs_fonciere.date_mutation, 'DD') AS day
            ";
            $groupByAndOrder .= ",
                month,
                day
            ";
        }elseif ($interval == self::$INTERVAL_MONTH){
            $selection .= ",
                TO_CHAR(valeurs_fonciere.date_mutation, 'MM') AS month
            ";
            $groupByAndOrder .= ",
                month
            ";
        }
        $query = "  SELECT
                        {$selection},
                        COUNT(valeurs_fonciere.date_mutation) AS number_of_sold
                    FROM
                        App:ValeursFonciere AS valeurs_fonciere
                    WHERE
                        lower(valeurs_fonciere.nature_mutation) = 'vente'
                        AND
                        valeurs_fonciere.date_mutation >= TO_DATE('{$start_date}', 'DD-MM-YYYY')
                        AND
                        valeurs_fonciere.date_mutation <= TO_DATE('{$end_date}', 'DD-MM-YYYY')
                    GROUP BY
                        {$groupByAndOrder}
                    ORDER BY
                        {$groupByAndOrder}";
        $sold_houses = $this->em->createQuery($query)->getResult();
        foreach ($sold_houses as &$sold_house){
            $sold_house['year'] = intval($sold_house['year']);
            if($interval == self::$INTERVAL_DAY){
                $sold_house['month'] = intval($sold_house['month']);
                $sold_house['day'] = intval($sold_house['day']);
            }elseif ($interval == self::$INTERVAL_MONTH){
                $sold_house['month'] = intval($sold_house['month']);
            }
            $sold_house['number_of_sold'] = floatval($sold_house['number_of_sold']);
        }
        $res = new JsonResponse(
            $sold_houses,
            Response::HTTP_OK
        );
        return $res;
    }
}
